@@ -1,127 +1,14 @@
-"""
-Provides easy read/write access to genomic tracks in a fashion that is independent from the underlying format. Requires Python 2.6 or higher.
-Currently the following formats are implemented:
-
-* `BioSQLite <http://bbcf.epfl.ch/twiki/bin/view/BBCF/SqLite>`_
-* `BED <http://genome.ucsc.edu/FAQ/FAQformat.html#format1>`_
-* `WIG <http://genome.ucsc.edu/goldenPath/help/wiggle.html>`_
-* `GFF <http://genome.ucsc.edu/FAQ/FAQformat.html#format3>`_
-* `GTF <http://genome.ucsc.edu/FAQ/FAQformat.html#format4>`_
-* `bedGraph <http://genome.ucsc.edu/goldenPath/help/bedgraph.html>`_
-* `bigWig <http://genome.ucsc.edu/goldenPath/help/bigWig.html>`_
-
-More formats can be added easily.
-
-############
-Installation
-############
-
-To install you should download the latest source code from GitHub, either by going to::
-
-    http://github.com/xapple/track
-
-and clicking on "Downloads", or by cloning the git repository with::
-
-    $ git clone https://github.com/xapple/track.git
-
-Once you have the source code, run::
-
-    $ cd track
-    $ sudo python setup.py install
-
-to install it. If you don't have permission to install it in the default directory, you can simply build the source in-place and use the package from the git repository::
-
-    $ python python setup.py build_ext --inplace
-
-###########
-Quick start
-###########
-
-To get access to the information contained inside already existing genomic tracks, you would do the following whatever the format of the track is::
-
-    import track
-    with track.load('tracks/rp_genes.bed') as rp:
-        data = rp.read('chr3')
-
-The ``data`` variable will now yield tuples containing genomic features on chromosome three. Typically, a feature consits of elements such as a start coordinate, a stop coordinate, a name, a score value, and a strand. Hence, when calling ``data.next()`` you will get objects looking something like this: ``(15, 20, 'Gene A', 3.0, 1)``.
-
-However, it is strongly advised to convert your tracks into the SQL format before working with them for better performance. If you do this, your text files will only be parsed once. The previous code becomes::
-
-    import track
-    track.convert('tracks/rp_genes.bed', 'tracks/rp_genes.sql')
-    with track.load('tracks/rp_genes.sql') as rp:
-        data = rp.read('chr3')
-
-Now, let's use our read query for computing something. For instance, the cumulative base coverage of all features on chromosome two can be calculated like this::
-
-    import track
-    with track.load('tracks/rp_genes.gff') as rp:
-        all_genes = rp.read('chr2')
-        base_coverage = sum([gene[1] - gene[0] for gene in all_genes])
-        # gene[1] is the end coordinate and gene[0] is the start coordinate
-
-The results coming from the ``read`` function can also be referenced by field name. Hence this code works as well::
-
-    import track
-    with track.load('tracks/rp_genes.gff') as rpgenes:
-        all_genes = rpgenes.read('chr2')
-        base_coverage = sum([gene['end'] - gene['start'] for gene in all_genes])
-
-To create a new track and then write to it, you should use the following::
-
-    import track
-    with track.new('tracks/rap1_peaks.sql') as mypeaks:
-        mypeaks.write('chr1', [(10, 20, 'Peak A', 0.0, 1)])
-
-To duplicate a chromosome inside the same track, you can use the following::
-
-    with track.load('tracks/copychrs.sql') as t:
-        t.write('chrY', t.read('chrX'))
-
-To make a new track from an old one, and invert the strand of every feature::
-
-    import track
-    def invert_strands(data):
-        for gene in data:
-            yield (gene[0], gene[1], gene[2], gene[3], gene[4] == 1 and -1 or 1)
-    with track.load('tracks/orig.sql') as orig:
-        with track.new('tracks/inverted.sql') as inverted:
-            for chrom in orig:
-                inverted.write(chrom, invert_strands(orig.read(chrom)))
-
-To convert a track from a format (e.g. BED) to an other format (e.g. GFF) you call the `track.convert` function::
-
-    import track
-    track.convert('tracks/rp_genes.bed', 'tracks/rp_genes.gff')
-
-If your track is in a format that is missing chromosome information (such as the length of every chromosome), you can supply an assembly name or a chromosome file::
-
-    import track
-    with track.load('tracks/yeast_genes.sql') as t:
-        # Specify the assembly
-        t.assembly = 'hg19'
-        # Or load a tab delimited file
-        t.load_chr_file('info/yeast.chr')
-
-To set the chromosome metadata or the track metadata you simply assign to that attribute::
-
-    import track
-    with track.load('tracks/scores.sql') as t:
-        t.chrmeta = ``{'chr1': {'length': 197195432}, 'chr2': {'length': 129993255}}``
-        t.info = {'datatype': 'signal', 'source': 'UCSC'}
-"""
-
 b'This module needs Python 2.6 or later.'
 
 # Special variables #
-__version__ = '1.0.0'
+__version__ = '1.1.0'
 __all__ = ['load', 'new', 'convert']
 
 # Other variables #
-formats = ('bed', 'wig', 'gff', 'gtf', 'bedGraph', 'bigWig')
+formats = ('sql', 'bed', 'wig', 'gff', 'gtf', 'bedgraph', 'bigwig', 'gzip', 'sga')
 
 # Built-in modules #
-import os, re, sqlite3
+import os, re, sqlite3, json
 from itertools import imap
 
 # Internal modules #
@@ -134,7 +21,7 @@ from track.common import check_path, check_file, empty_file, empty_sql_file, tem
 from track.common import JournaledDict, natural_sort, int_to_roman, roman_to_int
 from track.common import Color, pick_iterator_elements, get_next_item, is_gzip
 from track.common import if_url_then_get_url
-from track.genrep import Assembly
+from track.genomes import Assembly
 
 # Compiled modules #
 from track.pyrow import SuperRow
@@ -228,7 +115,7 @@ def convert(source, destination, assembly=None):
        :type  source: string
        :param destination: is the path to the track to be created.
        :type  destination: string
-       :param assembly: an optional GenRep compatible assembly name or id. Useful when the destination format needs to contain chromosome meta data and this is not available in the source file.
+       :param assembly: an optional compatible assembly name. Useful when the destination format needs to contain chromosome meta data and this is not available in the source file.
        :type  assembly: string
 
        :returns: the path to the track created (or a list of track paths in the case of multi-track files).
@@ -288,7 +175,7 @@ class Track(object):
     """
 
     def __init__(self, path, readonly=False, autosave=True, orig_path=None, orig_format=None):
-        """The track package is designed to be accesed via the 'load()' and 'new()'
+        """The track package is designed to be accessed via the 'load()' and 'new()'
            functions in order to create Track objects.
            Usually, the constructor is not called directly."""
         # Passed attributes #
@@ -344,16 +231,20 @@ class Track(object):
         if isinstance(key,int): return self.chromosomes[key]
         else: return self.read(key)
 
+    def __repr__(self):
+        """Called when evaluating ``print t``."""
+        return '<%s object on "%s">' % (self.__class__.__name__, self.path)
+
     #-----------------------------------------------------------------------------#
     @property
     def modified(self):
-        """*modified* is a boolean value which indicates if the track has been changed since it was opened. This value is set to False when you load a track and is set to True as soon, as you ``write``, ``rename`` or ``remove``. Changing the ``info`` or ``chrmeta`` attributes will also set this value to True."""
+        """A boolean value which indicates if the track has been changed since it was opened. This value is set to False when you load a track and is set to True as soon, as you ``write``, ``rename`` or ``remove``. Changing the ``info`` or ``chrmeta`` attributes will also set this value to True."""
         if self._modified or self.info.modified or self.chrmeta.modified: return True
         return False
 
     @property
     def fields(self):
-        """*fields* is a list the value types that each feature in the track will contain. For instance:
+        """A list the value types that each feature in the track will contain. For instance:
 
                ``['start', 'end', 'name', 'score', 'strand']``
 
@@ -372,13 +263,13 @@ class Track(object):
 
     @property
     def tables(self):
-        """Returns the complete list of SQL tables."""
+        """The complete list of SQL tables."""
         self._cursor.execute("select name from sqlite_master where type='table'")
         return [x[0].encode('ascii') for x in self._cursor.fetchall()]
 
     @property
     def chromosomes(self):
-        """*chromosomes* is a list of all available chromosome. For instance:
+        """A list of all available chromosome. For instance:
 
                ``['chr1, 'chr2', 'chr3', 'chr4', 'chr5', 'chrC', 'chrM']``
 
@@ -389,7 +280,7 @@ class Track(object):
         return chroms
 
     def _get_fields_of_table(self, chrom):
-        """Returns the list of fields for a particular table by querying the SQL for the complete list of column names"""
+        """Return the list of fields for a particular table by querying the SQL for the complete list of column names"""
         # Check the table exists #
         if not chrom in self.tables: return []
         # A pragma statement will implicitly issue a commit, don't use #
@@ -400,7 +291,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def cursor(self):
-        """*cursor* will create a new sqlite3 cursor object connected to the track database. You can use this attribute to make your own SQL queries and fetch the results. More information is available on the `sqlite3 documentation pages <http://docs.python.org/library/sqlite3.html>`_.
+        """Create a new sqlite3 cursor object connected to the track database. You can use this attribute to make your own SQL queries and fetch the results. More information is available on the `sqlite3 documentation pages <http://docs.python.org/library/sqlite3.html>`_.
 
         :returns: A new sqlite3 cursor object
 
@@ -418,7 +309,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def save(self):
-        """Stores the changes that were applied to the track on the disk. If the track was loaded from a text file such as 'bed', the file is rewritten with the changes included. If the track was loaded as an SQL file, the changes are committed to the database. Calling ``rollback`` will revert all changes to the track since the last call to ``save()``. By default, when the track is closed, all changes are saved.
+        """Store the changes that were applied to the track on the disk. If the track was loaded from a text file such as 'bed', the file is rewritten with the changes included. If the track was loaded as an SQL file, the changes are committed to the database. Calling ``rollback`` will revert all changes to the track since the last call to ``save()``. By default, when the track is closed, all changes are saved.
 
         :returns: None
 
@@ -455,7 +346,7 @@ class Track(object):
             raise Exception(message % (self.path, err))
 
     def _make_missing_tables(self):
-        """Makes sure every chromsome referenced in the chrNames table exists as a table in the database. Will create empty tables."""
+        """Make sure every chromosome referenced in the 'chrNames' table exists as a table in the database. Will create empty tables."""
         fields = self.fields or minimum_fields
         fields = ','.join(['"' + f + '"' + ' ' + sql_field_types.get(f, 'text') for f in fields])
         for chrom_name in sorted(self.chrmeta, key=natural_sort):
@@ -463,7 +354,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def rollback(self):
-        """Reverts all changes to the track since the last call to ``save()``.
+        """Revert all changes to the track since the last call to ``save()``.
 
         :returns: None
 
@@ -494,7 +385,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def close(self):
-        """Closes the current track. This method is useful when for some special reason you are not using the 'with ... as' form for loading tracks.
+        """Close the current track. This method is useful when for some special reason you are not using the ``with ... as``` form for loading tracks.
 
         :returns: None
 
@@ -521,7 +412,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def export(self, path, format=None):
-        """Exports the current track to a given format. A new file is created at the specified path. The current track object is unchanged
+        """Export the current track to a given format. A new file is created at the specified path. The current track object is unchanged
 
         :param path: is the path to track file to create.
         :type  path: string
@@ -550,7 +441,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def read(self, selection=None, fields=None, order='start,end'):
-        """Reads data from the track.
+        """Read data from the track.
 
         :param selection: A chromosome name, or a dictionary specifying a region, see below.
         :param fields: is an optional list of fields which will influence the length of the tuples returned and the way in which the information is returned. The default is to read every field available for the given chromosome. If the *track.fields* attribute is set, that will be used.
@@ -607,7 +498,7 @@ class Track(object):
             # Columns names in the table #
             available_fields = self._get_fields_of_table(chrom)
             # Fields attribute is set or not #
-            query_fields = ','.join([f in available_fields and f or py_field_types[f]().__repr__() for f in fields and fields or self._fields])
+            query_fields = ','.join([f in available_fields and f or py_field_types.get(f, str)().__repr__() for f in fields and fields or self._fields])
         ##### QUERY #####
         sql_command = "SELECT " + query_fields + " from '" + chrom + "'"
         # Add the where case #
@@ -628,7 +519,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def write(self, chromosome, data, fields=None):
-        """Writes data to a genomic file. Will write many feature at once into a given chromosome.
+        """Write data to a genomic file. Will write many feature at once into a given chromosome.
 
         :param chromosome: is the name of the chromosome on which one wants to write. For instance, if one is using the BED format this will become the first column, while if one is using the SQL format this will become the name of the table to be created.
         :type  chromosome: string
@@ -708,7 +599,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def insert(self, chromosome, feature):
-        """Inserts one feature into an existing chromosome.
+        """Insert one feature into an existing chromosome.
 
         :param chromosome: is the name of the chromosome into which one wants to insert.
         :type  chromosome: string
@@ -728,7 +619,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def remove(self, chromosome):
-        """Removes data from a given chromosome.
+        """Remove data from a given chromosome.
 
         :param chromosome: is the name of the chromosome that one wishes to delete or a list of chromosomes to delete.
         :type  chromosome: string
@@ -753,9 +644,51 @@ class Track(object):
             self._cursor.execute("DROP table '" + chromosome + "'")
             if chromosome in self.chrmeta: self.chrmeta.pop(chromosome)
 
+    #-----------------------------------------------------------------------------#
+    def delete_fields(self, fields):
+        """Remove the given fields from all chromosomes. This is equivalent to dropping full columns in the database.
+
+        :param fields: A list of fields such as ``['score','strand']``.
+        :type  fields: list
+
+        :returns: None.
+
+        ::
+
+            import track
+            with track.load('tracks/example.sql') as t:
+                print t.fields
+                t.delete_fields(['score','strand'])
+                print t.fields
+        """
+        # Check track attributes #
+        self._modified = True
+        if self.readonly: return
+        # SQLite doesn't support dropping columns directly #
+        sql_script = '''CREATE TABLE "%(chrom)s_tmp" (%(types)s);
+        INSERT INTO "%(chrom)s_tmp" SELECT %(names)s FROM "%(chrom)s";
+        DROP TABLE "%(chrom)s";
+        ALTER TABLE "%(chrom)s_tmp" RENAME TO "%(chrom)s";'''
+        # Loop on chromosomes #
+        for chrom in self:
+            current_fields = self._get_fields_of_table(chrom)
+            # Should anything change ? #
+            if not set(fields) & set(current_fields): continue
+            # What will be the fields now ? #
+            new_fields = [f for f in current_fields if f not in fields]
+            # Do we have anything left ? #
+            if not new_fields:
+                self.remove(chrom)
+                return
+            # Drop all the columns #
+            names = ','.join(['"' + f + '"' for f in new_fields])
+            types = ','.join(['"' + f + '"' + ' ' + sql_field_types.get(f, 'text') for f in new_fields])
+            custom_sql_script= sql_script % dict(chrom=chrom, names=names, types=types)
+            self._cursor.executescript(custom_sql_script)
+
     #-----------------------------------------------------------------------------#
     def rename(self, previous_name, new_name):
-        """Renames a chromosome from *previous_name* to *new_name*
+        """Rename a chromosome from *previous_name* to *new_name*
 
         :param previous_name: is the name of the chromosome that one wishes to rename.
         :type  previous_name: string
@@ -802,7 +735,8 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def search(self, query_dict, fields=None, chromosome=None, exact_match=False):
-        """Searches for parameters inside your tracks. You can specify several parameters.
+        """Search for parameters inside your track. You can specify several parameters.
+
         :param selection: list of the fields you want to have in the result (to insure that all result will have the same number of columns)
         :param query_dict: A dictionary specifying keys and values to search for. See examples.
         :type  query_dict: dict
@@ -837,9 +771,31 @@ class Track(object):
         cursor = self.cursor()
         return cursor.execute(query_str)
 
+    #-----------------------------------------------------------------------------#
+    def find_column_name(self, synonyms, table_name=None):
+        """Search for a field name in the database. This can be useful when a specific field name has several synonyms and you don't know which one your data source is using. If several names match, will only return the first match.
+
+        :param synonyms: Several names commonly used for the same thing.
+        :type  synonyms: list
+        :param table_name: Optionally, a specific chromosome to search on.
+        :type  table_name: list
+
+        :returns: An element of **synonyms** or an empty string if no match is found.
+
+        ::
+            with track.load('tracks/rp_genes.bed') as t:
+                name = t.find_column_name(['name', 'gene_name', 'gname', 'gene name'])
+                > gene_name
+        """
+        if table_name is not None: fields = self.fields
+        else:                      fields = self._get_fields_of_table(table_name)
+        for name in synonyms:
+            if name in self.fields: return name
+        return ''
+
     #-----------------------------------------------------------------------------#
     def count(self, selection=None):
-        """Counts the number of features or entries in a given selection.
+        """Count the number of features or entries in a given selection.
 
         :param selection: is the name of a chromosome, a list of chromosomes, a particular span or a list of spans. In other words, a value similar to the *selection* parameter of the *read* method. If left empty, will count every feature in a track
 
@@ -876,7 +832,7 @@ class Track(object):
 
     #-----------------------------------------------------------------------------#
     def ucsc_to_ensembl(self):
-        """Converts all entries of a track from the UCSC standard to the Ensembl standard effectively adding one to every start position.
+        """Convert all entries of a track from the UCSC standard to the Ensembl standard effectively adding one to every start position.
 
         :returns: None.
 
@@ -902,8 +858,8 @@ class Track(object):
         for chrom in self.chromosomes: self._cursor.execute("update '" + chrom + "' set start=start-1")
 
     #-----------------------------------------------------------------------------#
-    def get_score_vector(self, chromosome):
-        """Returns an iterable with as many elements as there are base pairs in the chromosomes specified by the *chromosome* parameter. Every element of the iterable is a float indicating the score at that position. If the track has no score associated, ones are inserted where features are present.
+    def get_full_score_vector(self, chromosome):
+        """Create an iterable with as many elements as there are base pairs in the chromosomes specified by the *chromosome* parameter. Every element of the iterable is a float indicating the score at that position. If the track has no score associated, ones are inserted where features are present.
 
         :param chromosome: is the name of the chromosome on which one wants to create a score vector from.
         :type  chromosome: string
@@ -914,15 +870,17 @@ class Track(object):
 
             import track
             with track.new('tmp/track.sql') as t:
-                scores = t.get_score_vector('chr1')
+                scores = t.get_full_score_vector('chr1')
         """
-        # Conditions #
-        if 'score' not in self.fields:
-            def add_ones(X):
-                for x in X: yield x + (1.0,)
-            data = add_ones(self.read(chromosome, ['start','end']))
-        else:
-            data = self.read(chromosome, ['start','end','score'])
+        # Check chromosome existence #
+        if chromosome not in self: return
+        chr_length = self.chrmeta[chromosome]['length'] if chromosome in self.chrmeta else None
+        # Call read #
+        fields = ['start','end','score'] if 'score' in self.fields else ['start','end']
+        data = self.read(chromosome, fields)
+        # Special function for tracks without score #
+        add_ones = lambda X: (tuple(x) + (1.0,) for x in X)
+        if 'score' not in self.fields: data = add_ones(data)
         # Initialization #
         last_end = 0
         x = (-1,0)
@@ -932,12 +890,56 @@ class Track(object):
             for i in xrange(x[0],     x[1]): yield x[2]
             last_end = x[1]
         # End piece #
-        if self.chrmeta.get(chromosome):
-            for i in xrange(x[1], self.chrmeta[chromosome]['length']): yield 0.0
+        if chr_length:
+            for i in xrange(x[1], chr_length): yield 0.0
+
+    #-----------------------------------------------------------------------------#
+    def get_partial_score_vector(self, chromosome, start, end):
+        """Create an iterable with as many elements as there are base pairs in the interval between *start* and *end*. Every element of the iterable is a float indicating the score at that position. If the track has no score associated, ones are inserted where features are present.
+
+        :param chromosome: is the name of the chromosome on which one wants to create a score vector from.
+        :type  chromosome: string
+        :param start: The base pair position where scores will start being read from. Defaults to 0.
+        :type  start: int
+        :param end: The base pair position where scores will stop being read from. Defaults to the length of the chromosome.
+        :type  end: int
+
+        :returns: an iterable yielding floats.
+
+        ::
+
+            import track
+            with track.new('tmp/track.sql') as t:
+                scores = t.get_score_vector('chr1')
+        """
+        # Check chromosome existence #
+        if chromosome not in self: return
+        # Call read #
+        selection = {'chr': chromosome, 'start': start, 'end': end}
+        fields = ['start','end','score'] if 'score' in self.fields else ['start','end']
+        data = self.read(selection, fields)
+        # Special function for tracks without score #
+        add_ones = lambda X: (tuple(x) + (1.0,) for x in X)
+        if 'score' not in self.fields: data = add_ones(data)
+        # Core loop #
+        x = (start, start)
+        for x in data:
+            if start >= x[1]: continue
+            if start < x[0]:
+                for i in xrange(start, x[0]): yield 0.0
+                start = x[0]
+            if end <= x[1]:
+                for i in xrange(start, end): yield x[2]
+                break
+            else:
+                for i in xrange(start,  x[1]): yield x[2]
+                start = x[1]
+        # End piece #
+        for i in xrange(x[1], end): yield 0.0
 
     #-----------------------------------------------------------------------------#
     def roman_to_integer(self, names=None):
-        """Converts the name of all chromosomes from the roman numeral standard to the arabic numeral standard. For instance, 'chrI' will become 'chr1' while 'chrII' will become 'chr2', etc.
+        """Convert the name of all chromosomes from the roman numeral standard to the arabic numeral standard. For instance, 'chrI' will become 'chr1' while 'chrII' will become 'chr2', etc.
 
         :param names: an optional dictionary specifying how to translate particular cases. Example: ``{'chrM':'chrQ', '2micron':'chrR'}``
         :type  names: dict
@@ -959,7 +961,7 @@ class Track(object):
         for chrom in self: self.rename(chrom, convert(chrom))
 
     def integer_to_roman(self, names=None):
-        """Converts the name of all chromosomes from the arabic numeral standard to the roman numeral standard. For instance, 'chr1' will become 'chrI' while 'chr2' will become 'chrII', etc.
+        """Convert the name of all chromosomes from the arabic numeral standard to the roman numeral standard. For instance, 'chr1' will become 'chrI' while 'chr2' will become 'chrII', etc.
 
         :param names: an optional dictionary specifying how to translate particular cases. Example: ``{'chrQ':'chrM', 'chrR':'2micron'}``
         :type  names: dict
@@ -983,7 +985,7 @@ class Track(object):
     #--------------------------------------------------------------------------#
     @property
     def info(self):
-        """*info* is a dictionary of meta data associated to the track (information like the source, etc). For instance:
+        """A dictionary of meta data associated to the track (information like the source, etc). For instance:
 
               ``{'datatype': 'signal', 'source': 'SGD', 'orig_name': 'splice_sites.bed'}``
         """
@@ -994,7 +996,7 @@ class Track(object):
         self._info.overwrite(value)
 
     def _info_read(self):
-        """Populates the *self.info* attribute with information found in the 'attributes' table."""
+        """Populate the *self.info* attribute with information found in the 'attributes' table."""
         if not 'attributes' in self.tables: return
         # Make a dictionary directly from the table #
         query = self._cursor.execute('SELECT key, value from "attributes"')
@@ -1003,7 +1005,7 @@ class Track(object):
         self.info.modified = False
 
     def _info_write(self):
-        """Rewrites the 'attributes' table so that it reflects the contents of the *self.info* attribute."""
+        """Rewrite the 'attributes' table so that it reflects the contents of the *self.info* attribute."""
         if self.readonly: return
         self._cursor.execute('DROP table IF EXISTS "attributes"')
         if not self.info: return
@@ -1014,12 +1016,13 @@ class Track(object):
 
     @property
     def datatype(self):
-        """Giving a datatype to your track is optional. The default datatype is ``None``. Other possible datatypes are ``features``, ``signal`` or ``relational``. Changing the datatype imposes some conditions on the entries that the track contains. This attribute is stored inside the *info* dictionary::
+        """Giving a datatype to your track is optional. The default datatype is ``None``. Other possible datatypes are ``features``, ``signal`` or ``relational``. Changing the datatype imposes some conditions on the entries that the track contains. This attribute is stored inside the *info* dictionary.
+
+        ::
 
             import track
             with track.new('tmp/track.sql') as t:
                 t.datatype = 'signal'
-
         """
         return self.info.get('datatype', None)
 
@@ -1031,7 +1034,7 @@ class Track(object):
 
     @property
     def name(self):
-        """Giving a name to your track is optional. The default name is ``Unnamed``. This attribute is stored inside the *info* dictionary."""
+        """Giving a name to your track is optional. The default name is the filename. This attribute is stored inside the *info* dictionary."""
         return self.info.get('name', os.path.basename(self.path))
 
     @name.setter
@@ -1041,7 +1044,7 @@ class Track(object):
     #-----------------------------------------------------------------------------#
     @property
     def chrmeta(self):
-        """*chrmeta* is an attribute containing extra chromosomal meta data such chromosome length information. *chrmeta* is a dictionary where each key is a chromosome names. For instance:
+        """Contains extra chromosomal meta data such as chromosome length information. *chrmeta* is a dictionary where each key is a chromosome names. For instance:
 
              ``{'chr1': {'length': 197195432}, 'chr2': {'length': 129993255}}``
 
@@ -1059,7 +1062,7 @@ class Track(object):
         self._chrmeta.overwrite(value)
 
     def _chrmeta_read(self):
-        """Populates the self.chrmeta attribute with information found in the 'chrNames' table."""
+        """Populate the self.chrmeta attribute with information found in the 'chrNames' table."""
         # If the table doesn't exist, just use the names
         if not 'chrNames' in self.tables:
             dictionary = dict([(chrom, dict()) for chrom in self])
@@ -1079,14 +1082,18 @@ class Track(object):
         self.chrmeta.modified = False
 
     def _chrmeta_write(self):
-        """Rewrites the 'chrNames' table so that it reflects the contents of the self.chrmeta attribute."""
+        """Rewrite the 'chrNames' table so that it reflects the contents of the self.chrmeta attribute."""
         if self.readonly: return
         self._cursor.execute('DROP table IF EXISTS "chrNames"')
         if not self.chrmeta: return
         # Rows are the chromosome names #
         # [{'name': 'chr1', 'length': 1000}, {'name': 'chr2', 'length': 2000}]
         rows = [dict([('name', chrom)] + [(k,v) for k,v in self.chrmeta[chrom].items()]) for chrom in self.chrmeta]
-        self._cursor.execute('CREATE table "chrNames" ("name" text, "length" integer)')
+        # Columns are the field names #
+        # ['name', 'length']
+        columns = rows[0].keys()
+        fields = ','.join(['"' + field + '"' + ' ' + sql_field_types.get(field, 'text') for field in columns])
+        self._cursor.execute('CREATE table "chrNames" (' + fields + ')')
         for r in sorted(rows, key=lambda x: natural_sort(x['name'])):
             question_marks = '(' + ','.join(['?' for x in r.keys()]) + ')'
             column_names   = '(' + ','.join(['"' + k + '"' for k in r.keys()]) + ')'
@@ -1115,7 +1122,7 @@ class Track(object):
 
     @property
     def assembly(self):
-        """Giving an assembly to your track is optional. However, if you set this variable for your track, you should input with a GenRep compatible assembly name or id. Doing so, will download the relevant information from GenRep, set the *chrmeta* attribute and rename all the chromosome to their canonical names if a correspondence is found. You can also call ``guess_assembly()``. This attribute is also stored inside the *info* dictionary.
+        """Giving an assembly to your track is optional. However, if you set this variable for your track, you should input a valid assembly name such as 'sacCer2'. Doing so will set the *chrmeta* attribute and rename all the chromosome to their canonical names if a correspondence is found. You can also call ``guess_assembly()``. This attribute is also stored inside the *info* dictionary.
 
         ::
 
@@ -1130,8 +1137,8 @@ class Track(object):
     def assembly(self, value):
         # Get the assembly #
         assembly = Assembly(value)
-        # Check that it is valid #
-        if not assembly.name: return
+        # Add the attribute #
+        self.info['assembly'] = value
         # Check if the tables need renaming or deleting #
         for orig_name in self.chromosomes:
             cannonical_name = assembly.guess_chromosome_name(orig_name)
@@ -1139,8 +1146,46 @@ class Track(object):
             else: self.remove(orig_name)
         # Add the chrmeta #
         self.chrmeta = assembly.chrmeta
-        # Add the attribute #
-        self.info['assembly'] = assembly.name
+
+    #-----------------------------------------------------------------------------#
+    def aggregated_read(self, selection, base_fields, order_by=None):
+        """
+        Read the track with the *base_fields*, and aggregate all supplementary fields
+        found in a single string.
+
+        :param selection: The **selection** argument is passed to the **read** function.
+        :type selection: list
+        :param base_fields: the columns you don't want to aggregate.
+        :type base_fields: list
+        :param order_by: The **order_by** argument is passed to the **read** function.
+        :type order_by: list
+
+        :returns: a generator object yielding rows.
+
+        ::
+
+            >> import track
+            >> t = track.load('tmp/track.sql')
+            >> t.fields
+            ['start', 'stop', 'score', 'name', 'type', 'strand']
+            >> for feature in t.read('chr1'): print feature
+            [1,  10, 2.2, 'tc36', 'transcript', '+']
+            [1,  10, 3.4, 'tc36', 'transcript', '-']
+            [11, 12, 3.4, 'tc37', 'transcript', '+']
+            >> data = t.aggregated_read('chr1', ('start', 'stop', 'score'))
+            >> for feature in data: print feature
+            [1,  10, 2.2, "{name :'tc36', type:'transcript', strand:'+'}"]
+            [1,  10, 3.4, "{name :'tc36', type:'transcript', strand:'-'}"]
+            [11, 12, 3.4, "{name :'tc37', type:'transcript', strand:'+'}"]
+        """
+        base_fields = tuple(base_fields)
+        track_fields = self.fields
+        supplementary_fields = tuple(set(track_fields) - set(base_fields))
+        blen = len(base_fields)
+        ll = range(blen)
+        for feature in self.read(selection, base_fields + supplementary_fields, order=order_by):
+            d = [{field : feature[field]} for field in supplementary_fields if field in feature.keys()]
+            yield [feature[i] for i in ll] + [json.dumps(d)]
 
 ################################################################################
 class FeatureStream(object):
